#!/bin/bash
# Helper script for generating developer documentation in
# api/ and api.rst

TOPDIR=../..
DOCDIR=$TOPDIR/doc/src
SRCDIR=$TOPDIR/cli/lmi
WITH_COMMANDS=${WITH_COMMANDS:-0}
JUST_SCRIPTS=${JUST_SCRIPTS:-0}

function get_modules() {
    pushd "$SRCDIR/$1" >/dev/null
    find -type f -a -name '*.py' | \
        sed -e 's:/__init__\.py$::' -e 's:^./::' | \
        grep -v "^\.$" | \
        grep -v '\(^\|/\)_' | \
        sort -u
    popd >/dev/null
}

function get_module_name() {
    path=`sed -e "s:^.*${SRCDIR}/\(.*\)$:\1:" -e "s:/:.:g"`
    basename -s .py "$path"
}

function get_version_string() {
    pushd $TOPDIR/cli >/dev/null
    setup=setup.py
    [ $JUST_SCRIPTS = 1 ] && setup=setup-scripts.py
    out=`python $setup --version`
    git_version=`git describe HEAD 2>/dev/null`
    echo -n "\`\`$out\`\`"
    if [ -n "$git_version" ]; then
        echo ", git: \`\`$git_version\`\`"
    fi
    popd >/dev/null
}

function get_scripts_version_string() {
    cat $OPENLMI_SCRIPTS_DIR/VERSION 2>/dev/null
}

declare -a gendirs
[ $JUST_SCRIPTS = 0 ] && gendirs+=("shell")
gendirs+=("scripts/common")
for dir in ${gendirs[@]}; do
    [ -e $DOCDIR/api/$dir ] || mkdir -p $DOCDIR/api/$dir
    [ -e $DOCDIR/api/$dir ] || mkdir -p $DOCDIR/api/$dir
done

# api.rst header
if [[ $JUST_SCRIPTS = 0 ]]; then
    cat >$DOCDIR/api.rst <<_EOF_
OpenLMI Tools API reference
===========================

This is a generated documentation from *OpenLMI Tools* sources.

Generated from version: $(get_version_string)

.. only:: html

    **Contents:**

.. toctree::
    :maxdepth: 2

    api/shell
    api/scripts
_EOF_
else
    cat >$DOCDIR/api.rst <<_EOF_
OpenLMI Scripts API reference
=============================

This is a generated documentation from *OpenLMI Scripts* sources.

Generated from version: $(get_version_string)

.. only:: html

    **Contents:**

.. toctree::
    :maxdepth: 2

    api/scripts
_EOF_
fi

cat >$DOCDIR/api/shell.rst <<_EOF_
LMIShell API reference
======================

This is a generated documentation from *LMIShell*'s sources.

Generated from version: $(get_version_string)

.. only:: html

    **Contents:**

.. toctree::
    :maxdepth: 2

_EOF_

cat >$DOCDIR/api/scripts.rst <<_EOF_
.. _openlmi-scripts-python:

LMI Scripts API reference
=========================

This is a generated documentation from *OpenLMI Scripts* sources. This covers
everything under :py:mod:\`lmi.scripts\` namespace.

:py:mod:\`lmi.scripts.common\` package provides useful functionality for script
development. Various scripts share this directory in order to provide
command-line interface through *LMI metacommand*.

Generated from version: $(get_version_string)

.. ifconfig:: with_commands

    Scripts version: $(get_scripts_version_string)

.. only:: html

    **Contents:**

.. toctree::
    :maxdepth: 1

    scripts/common
_EOF_

cat >$DOCDIR/api/scripts/common.rst <<_EOF_
LMI Scripts common library reference
====================================

This library builds on top of *LMIShell*'s functionality. It provides various
utilities and wrappers for building command-line interfaces to *OpenLMI
Providers*.

Generated from version: $(get_version_string)

-------------------------------------------------------------------------------

**Exported members:**

.. automodule:: lmi.scripts.common
    :members:
    
-------------------------------------------------------------------------------

**Submodules:**

.. toctree::
    :maxdepth: 1

_EOF_

if [[ $JUST_SCRIPTS = 0 ]]; then
    # Generate api/shell/*.rst
    for module_path in `get_modules shell`; do
        if [[ "$module_path" =~ ^compat ]]; then
            continue
        fi
        module=`echo "$module_path" | get_module_name`
        len=`echo $module | wc -c`
        len=$(($len-1))
        underline=""
        for i in `seq $len`; do underline="=$underline"; done
        echo >>$DOCDIR/api/shell.rst "    shell/$module"
        cat >$DOCDIR/api/shell/${module}.rst <<-_EOF_
	$module
	$underline
	.. automodule:: lmi.shell.$module
	    :members:
	_EOF_
    done
fi

# Generate api/scripts/common/*.rst
for module_path in `get_modules scripts/common`; do
    module=`echo "$module_path" | get_module_name`
    len=`echo $module | wc -c`
    len=$(($len-1))
    underline=""
    for i in `seq $len`; do underline="=$underline"; done
    echo >>$DOCDIR/api/scripts/common.rst "    common/$module"
    cat >$DOCDIR/api/scripts/common/${module}.rst <<-_EOF_
	$module
	$underline
	.. automodule:: lmi.scripts.common.$module
	    :members:
	_EOF_
done

# Add commands api reference to api/scripts.rst
if [[ $WITH_COMMANDS = 1 ]]; then
    if [ -z "$COMMANDS" ]; then
        COMMANDS=`find $OPENLMI_SCRIPTS_DIR/commands \
            -mindepth 1 -maxdepth 1 -type d | sort | xargs basename -a`
    fi

    for cmd in $COMMANDS; do
        link="api/scripts/$cmd"
        echo >>$DOCDIR/api/scripts.rst "    ../scripts/commands/$cmd/python"
    done
fi
